/*
 * SPDX-License-Identifier: Apache-2.0
 */

'use strict';

const { FileSystemWallet, Gateway } = require('fabric-network');
const path = require('path');

const ccpPath = path.resolve(__dirname, '.', 'network_profile.json');


const user_name = "lion";
const CHANNEL_NAME  = "appchannel";
const CHAIN_CODE_NAME = "carcc";
const CHAIN_CODE_FUNCTION_NAME = "listCars";

// const user_name = "lion";
// const CHANNEL_NAME  = "appchannel";
// const CHAIN_CODE_NAME = "testcc";
// const CHAIN_CODE_FUNCTION_NAME = "query";


async function main() {
    try {

        // Create a new file system based wallet for managing identities.

        

        const walletPath = path.join(process.cwd(), 'wallet');
        const wallet = new FileSystemWallet(walletPath);
        console.log(`Wallet path: ${walletPath}`);

        // Check to see if we've already enrolled the user.
        const userExists = await wallet.exists(user_name);
        if (!userExists) {
            console.log('An identity for the user "user1" does not exist in the wallet');
            console.log('Run the registerUser.js application before retrying');
            return;
        }

        // Create a new gateway for connecting to our peer node.
        const gateway = new Gateway();
    
        await gateway.connect(ccpPath, { wallet, identity: user_name, discovery: { enabled: false, asLocalhost: false } });
        

        // Get the network (channel) our contract is deployed to.
        const network = await gateway.getNetwork(CHANNEL_NAME);

        // Get the contract from the network.
        const contract = network.getContract(CHAIN_CODE_NAME);


        // Evaluate the specified transaction.
        // queryCar transaction - requires 1 argument, ex: ('queryCar', 'CAR4')
        // queryAllCars transaction - requires no arguments, ex: ('queryAllCars')
        
        const result = await contract.evaluateTransaction(CHAIN_CODE_FUNCTION_NAME);
        //const result = await contract.evaluateTransaction(CHAIN_CODE_FUNCTION_NAME,"b");
        //const result = await contract.evaluateTransaction(CHAIN_CODE_FUNCTION_NAME,"a");
        console.log(`Transaction has been evaluated, result is: ${result.toString()}`);
        process.exit(1);

    } catch (error) {
        console.error(`Failed to evaluate transaction: ${error}`);
        //process.exit(1);
    }
}

main();
